﻿using CoreAudio;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FocusVolumeControl
{
	public class ActiveAudioSessionWrapper
	{
		public string DisplayName { get; set; }
		public string ExecutablePath { get; set; }
		private List<SimpleAudioVolume> Volume { get; } = new List<SimpleAudioVolume>();

		public string Icon { get; set; }

		public bool Any()
		{
			return Volume.Any();
		}
		public int Count => Volume.Count;

		public void AddVolume(SimpleAudioVolume volume)
		{
			Volume.Add(volume);
		}

		public void ToggleMute()
		{
			//when all volumes are muted, Volume.All will return true
			//so we swap from muted to false (opposite of Volume.All)

			//when any volumes are unmuted, Volume.All will return false
			//so we set muted to true (opposite of Volume.All)

			var muted = Volume.All(x => x.Mute);

			Volume.ForEach(x => x.Mute = !muted);
		}

		public bool? GetMuted()
		{
			var muted = Volume.All(x => x.Mute);
			var unmuted = Volume.All(x => !x.Mute);

			if(muted == !unmuted)
			{
				return muted;
			}
			return null;
		}

		public void IncrementVolumeLevel(int step, int ticks)
		{
			//if you have more than one volume. they will all get set based on the first volume control
			var level = Volume.FirstOrDefault()?.MasterVolume ?? 0;

			level += (0.01f * step) * ticks;
			level = Math.Max(level, 0);

			Volume.ForEach(x => x.MasterVolume = level);
		}

		public int GetVolumeLevel()
		{
			var level = Volume.FirstOrDefault()?.MasterVolume ?? 0;
			return (int)(level * 100);
		}


	}
}
